#!/usr/bin/python

from matplotlib.mlab import bivariate_normal
#from matplotlib.mlab import meshgrid
from numpy import meshgrid
from pylab import *
import mpfit2 as mpfit
#from numarray import cos, sin, exp
from numpy import cos, sin, exp
#import numarray as na
import numpy as na
#from numarray import linear_algebra as la
#from numpy import linear_algebra as la
from random import randrange
from copy import deepcopy
#from scipy import optimize
import pdb

def model(p,fjac=None,x=None,y=None,err=None):
	
	base,amp,sigmax,sigmay,mux,muy = p

	model = base + amp * exp(-0.5*(((x[:,0]-mux)**2)/(sigmax**2) + ((x[:,1]-muy)**2)/(sigmay**2))) 
	status = 0
	return([status, (y-model)/err])
	
def base2DGauss(p,fjac=None,x=None,y=None,err=None):
    """
    NAM: baseDouble2DGauss (method)
    DES: derive 2D Gauss + first order basesurface 
    HIS: MA040515  initial version
    """
    
    rX=(x[:,0]-p[3])*cos(p[1])-(x[:,1]-p[5])*sin(p[1])
    rY=(x[:,0]-p[3])*sin(p[1])+(x[:,1]-p[5])*cos(p[1])
    model=p[0]*exp(-0.5*((rX/p[2])**2+(rY/p[4])**2)) + p[6]
    status = 0
    return([status, (y-model)/err])  

def detStartParaBase2DGauss(x,y,z): 
    """
    the following calculates start parameters
    """
    p=[]
    # gets the index of the maximum of z
    iMax=na.argmax(z)
    # gets the x coordinate of the maximum
    cX=x[iMax]
    # gets the y coordinate of the maximum
    cY=y[iMax]
    # bX  sigma is ~5 microns
    bX=750.
    bY=750.
    # here it is assumed that the base of the gaussian is an ellipse whose major axis inclination is ang
    # the ellipse is centred on cX, cY
    ang=0.1
##     # the following rotate the system of coordinates about the origin by an angle ang using the transformation
##     # x = X*cos(alpha) - Y*sin(alpha),
##     # y = X*sin(alpha) + Y*cos(alpha).
##     # 
##     # Then, because the new coordinate axes are parallel to the major and 
##     # minor axes of the ellipse, the equation of the ellipse has the form
##     # 
##     #    A*X^2 + C*Y^2 + D*X + E*Y + F = 0   (A*C > 0)
##     rX=(x-cX)*cos(ang)-(y-cY)*sin(ang)
##     rY=(x-cX)*sin(ang)+(y-cY)*cos(ang)
##     e=exp(-bX*rX**2-bY*rY**2)
##     #from here, we assume we know e. Then the new equation to solve is p[0] + p[6] e
##     # in order to get p[0], p[6]
    
##     m  = na.zeros((2,2))*1.
##     m[0,0] = len(z)
##     m[0,1] = m[1,0] = sum(e)
##     m[1,1] = sum(e**2.)
    
##     print "m = ", m
##     detm = la.determinant(m)
##     print  "matrix determinant = ", detm
##     inv_m = la.inverse(m)
##     print "inv_m = ", inv_m
    
##     v = na.zeros(2)*1.
##     v[0] = sum(z)
##     v[1] = sum(z*e)
    
##     print "v = ", v
##     p0, p6 = na.dot(inv_m, v)
    
    #guess amp is highes value
    #p0 = z.max() - z.min()
    p0 = na.max(z) - na.min(z)
    #guess that the base is the lowest value
    #p6 = z.min()
    p6 = na.min(z)
    p.append(p0)   # p[0]  amplitude
    p.append(ang)  # p[1]  angle
    p.append(bX)   # p[2]  sigmax
    p.append(cX)   # p[3]  meanx
    p.append(bY)   # p[4]  sigmay
    p.append(cY)   # p[5]  meany
    p.append(p6)   # p[6]  base
    print "p = ",p
    return p

def gauss_fit(intensity=0,x_pos=0,y_pos=0):
    #pdb.set_trace()
    x = na.zeros(len(intensity.flat))
    y = na.zeros(len(intensity.flat))
    z = intensity.flat
	
    for i in range(len(y_pos)):
        for j in range(len(x_pos)):
            x[i*len(x_pos)+j] = x_pos[j]
            y[i*len(x_pos)+j] = y_pos[i]
    p=detStartParaBase2DGauss(x,y,z)
    #parinfo=[{'value':0.,'mpprint':1}]*len(p)
    parinfo = [deepcopy({'value':0.,'mpprint':1,'fixed':0}) for i in range(len(p))]
    for i in range(len(p)): parinfo[i]['value']=p[i]
    xy=na.transpose(na.reshape(na.concatenate((x,y)),(2,na.shape(x)[0])))
    err = 1.0
    fa={'x':xy, 'y':z, 'err':err}
    m=mpfit.mpfit(base2DGauss,p,parinfo=parinfo,functkw=fa,quiet=0,\
                  ftol=1.e-10, xtol=1.e-10, gtol=1.e-10)
	
    print 'fit mparams: ', m.params	
    		
    p = m.params
    base = p[6]
    amp = p[0]
    sigmax = p[2]
    sigmay = p[4]
    mux = p[3]
    muy = p[5]
    #for the fit use more data points, using delta
    delta = (x_pos[1]-x_pos[0])/5.
    x_fit = na.arange(x_pos.min(),x_pos.max()+delta,delta)
    y_fit = na.arange(y_pos.min(),y_pos.max()+delta,delta)
    x_fit_mesh,y_fit_mesh = meshgrid(x_fit,y_fit) 
##     intensity_fit = base + amp*exp(-0.5*(((x_fit_mesh-mux)**2)/(sigmax**2) + ((y_fit_mesh-muy)**2)/(sigmay**2)))

    rX = (x_fit_mesh-p[3])*cos(p[1])-(y_fit_mesh-p[5])*sin(p[1])
    rY = (x_fit_mesh-p[3])*sin(p[1])+(y_fit_mesh-p[5])*cos(p[1])
##     intensity_fit = p[0]*exp(-p[2]*rX**2-p[4]*rY**2) +  p[6]
    intensity_fit = p[0]*exp(-0.5*((rX/p[2])**2+(rY/p[4])**2)) + p[6]

    corner_delta = (x_pos[1]-x_pos[0])*0.5
    mparams = [p[6],p[0],p[2],p[4],p[3],p[5]]
    
    return intensity_fit,mparams

"""
# the following functions are downloaded online, and modified
    def gaussian(base,height, center_x, center_y, width_x, width_y):
	width_x = float(width_x)
	width_y = float(width_y)
	return lambda x,y: height*exp(
                -(((center_x-x)/width_x)**2+((center_y-y)/width_y)**2)/2)+base

    def moments(data):
        base = data.min()
        total = data.sum()
	X, Y = na.indices(data.shape)
	x = (X*data).sum()/total
	y = (Y*data).sum()/total
	col = data[:, int(y)]
	width_x = na.sqrt(abs((na.arange(col.size)-y)**2*col).sum()/col.sum())
	row = data[int(x), :]
	width_y = na.sqrt(abs((na.arange(row.size)-x)**2*row).sum()/row.sum())
	height = data.max()
	return base, height, x, y, width_x, width_y

    def fitgaussian(intensity=0,x_pos=0,y_pos=0):
        params = moments(intensity)
	errorfunction = lambda p: ravel(gaussian(*p)(*indices(intensity.shape)) -
                                 intensity)
	p, success = optimize.leastsq(errorfunction, params)
	
	xmin = x_pos.min()
	ymin = y_pos.min()
	xstep = x_pos[1] - x_pos[0]
	ystep = y_pos[1] - y_pos[0]

	params = [p[0],p[1],p[4]*xstep,p[5]*ystep,p[2],p[3]]
	return p
"""

